import { GGcommand } from "../../GCC"
import { setBRUSH } from "../../main";
import { Brush } from "../../worldFab/Brush";

export class cmd_Brush extends GGcommand {
    static name = ["brush", "br"];
    static arguments = ["<ball / cylinder / cyl / cone / cube / pattern / pat>", "<radius:number>", "<block_name>", "<block_state:array>{optional}", "replace", "<mask_block> <mask_block_state:array>{optional} +"];
    static description = `Builds the shape out of a specified block type, while the Brush item is used. To use the default block_state, simple add empty square brackets[]. To only replace certain blocks, use the replace keyword followed by the block and block_state. These mask blocks can be extended indefinitely.\nExample:\n    !br ball 3 stone \n    !br cyl 4 wood ["wood_type"="acacia","stripped_bit"=true]\n    !br cone 7 orange_concrete [] replace sponge ["sponge_type"="wet"] grass []\n    !br pat 7 stone [] replace sponge ["sponge_type"="wet"]`;

    static execute(args=[], sender) {
        let brushArgs = args;
        let maskStates = [];
        if (brushArgs.length > 4) {
            if (!brushArgs.includes("replace")) {
                throw new Error('Command missing keyword "replace"');
            }

            let maskArr = brushArgs.slice(5, brushArgs.length);
            for (let i = 0; i < maskArr.length; i += 2) {
                let pair = maskArr.slice(i, i + 2);
                if (pair.length < 2 || pair[1] == "" || pair[1] == "[]") 
                    pair[1] = "defaultState";
                
                maskStates.push({ block: pair[0], blockData: pair[1] });
            }
        }
        else 
            maskStates = null;


        if (brushArgs.length == 3)
            brushArgs.push("defaultState");
        else if (brushArgs.length < 3) {
            throw new Error("Too few arguments provided");
        }

        if (brushArgs[3] == "" || brushArgs[3] == "[]")
            brushArgs[3] = "defaultState";
        
        setBRUSH(new Brush(sender, brushArgs[0], brushArgs[1], brushArgs[2], brushArgs[3], maskStates));
    }
}