import { system, Vector } from "@minecraft/server";
import { GGcommand } from "../../GCC"

import { notify } from "../../other/helpers";
import { Brush } from "../../worldFab/Brush";

export class cmd_GenShapeAt extends GGcommand {
    static name = ["genShapeAt"];
    static arguments = ["<ball / cylinder / cyl / cone / cube>", "<radius:number> / <dimension:array>{radius, height}", "<block_name>", "<block_state:array>{optional}", "replace", "<mask_block> <mask_block_state:array>{optional} +"];
    static description = `Generate a primitive shape at the block the player is looking at (same as brush). Only cylinder / cyl supports [radius,height]. \nExample:\n    !genShapeAt ball 10 stone\n    !genShapeAt cylinder [5, 10] wood ["wood_type"="birch","stripped_bit"=true]\n    !genShapeAt cone 7 orange_concrete [] replace sponge ["sponge_type"="wet"] grass []`;
    
    static shouldTP = false;

    static execute(args=[], sender) {
        let brushArgs = args;
        let maskStates = [];
        if (brushArgs.length > 4) {
            if (!brushArgs.includes("replace")) {
                throw new Error(`Command missing keyword "replace"`);
            }

            let maskArr = brushArgs.slice(5, brushArgs.length);
            for (let i = 0; i < maskArr.length; i += 2) {
                let pair = maskArr.slice(i, i + 2);
                if (pair.length < 2 || pair[1] == "" || pair[1] == "[]") 
                    pair[1] = "defaultState";
                
                maskStates.push({ block: pair[0], blockData: pair[1] });
            }
        }
        else 
            maskStates = null;


        if (brushArgs.length == 3)
            brushArgs.push("defaultState");
        else if (brushArgs.length < 3) {
            throw new Error("Too few arguments provided");
        }

        if (brushArgs[3] == "" || brushArgs[3] == "[]")
            brushArgs[3] = "defaultState";

        const dimension = brushArgs[1];
        const tempBrush = new Brush(sender, brushArgs[0], brushArgs[1], brushArgs[2], brushArgs[3], maskStates, true);

        const pos = this.getPlaceTarget(sender);
        if (pos == null)
            return;
        
        tempBrush.build(pos, parseInt(dimension[1]) || 0);
        
        // tp the player out of the shape they just spawned
        if (!brushArgs.includes("replace") && this.shouldTP) {
            const offset = brushArgs[0] == "cone" ? parseInt(dimension[1] || dimension[0]) : 0;
            system.run(() => sender.teleport(Vector.add(sender.location, new Vector(0, parseInt(dimension[1] || dimension[0]) + 1 + offset, 0))));
        }

        notify(`Generated Shape Successfully`);
    }

    static getPlaceTarget(sender) {
        const hit = sender.getBlockFromViewDirection({ maxDistance: 488 });
        if (hit != undefined)
            return hit.block.location;
        else {
            notify("Target out of reach", "warn");
            return null;
        } 
    }
}