import { BlockPermutation } from "@minecraft/server";
import { GGcommand } from "../../GCC"
import { SELECTION } from "../../main";
import { BlockStateStringToObject, formatObject, notify } from "../../other/helpers";

export class cmd_Fill extends GGcommand {
    static name = ["fill"];
    static arguments = ["<block_name>", "<block_state:array>{optional}", "replace", "<mask_block> <mask_block_state:array>{optional} +"];
    static description = "Remove all blocks in selected area";

    static execute(args=[], sender) {
        let fillArgs = args;
        let maskStates = [];
        if (fillArgs.length > 2) {
            if (!fillArgs.includes("replace")) {
                throw new Error(`Command missing keyword "replace"`);
            }

            let maskArr = fillArgs.slice(3, fillArgs.length);
            for (let i = 0; i < maskArr.length; i += 2) {
                let pair = maskArr.slice(i, i + 2);
                if (pair.length < 2 || pair[1] == "" || pair[1] == "[]") 
                    pair[1] = "defaultState";
                
                maskStates.push({ block: pair[0], blockData: pair[1] });
            }
        }
        else 
            maskStates = null;

        if (fillArgs.length == 1)
            fillArgs.push("defaultState");
        else if (fillArgs.length < 1) {
            throw new Error("Too few arguments provided");
        }

        if (fillArgs[1] == "" || fillArgs[1] == "[]")
            fillArgs[1] = "defaultState";

        if (maskStates != null)
            notify(maskStates.map(ms => formatObject(ms)));
        let block = fillArgs[0];
        let blockData = BlockStateStringToObject(fillArgs[1]) || {};
        let mask = null;
        if (maskStates != null) {
            mask = maskStates.map(m => {
                return {
                    block: m.block,
                    blockData: BlockStateStringToObject(m.blockData)
                };
            });
        }
        SELECTION.fillSelection(sender, BlockPermutation.resolve(block, blockData), mask);
    }
}