import { EquipmentSlot, system, world } from "@minecraft/server";
import { addCommands } from "./GCC";
import { cmd_AutoKillItems } from "./commands/settings/cmd_AutoKillItems";
import { cmd_AllowBuildClipping } from "./commands/settings/cmd_AllowBuildClipping";
import { cmd_TestArray } from "./commands/cmd_TestArray";
import { cmd_Brush } from "./commands/brush/cmd_Brush";
import { cmd_Shape } from "./commands/brush/cmd_Shape";
import { cmd_Block } from "./commands/brush/cmd_Block";
import { cmd_Mask } from "./commands/brush/cmd_Mask";
import { cmd_Size } from "./commands/brush/cmd_Size";
import { cmd_BlockState } from "./commands/brush/cmd_BlockState";
import { cmd_GenShapeAt } from "./commands/genShape/cmd_GenShapeAt";
import { cmd_GenShape } from "./commands/genShape/cmd_GenShape";
import { Selection } from "./worldFab/Selection";
import { Brush } from "./worldFab/Brush";

import { cmd_Pos } from "./commands/selection/cmd_Pos";
import { cmd_Delete } from "./commands/selection/cmd_Delete";
import { cmd_Copy } from "./commands/selection/cmd_Copy";
import { cmd_Paste } from "./commands/selection/cmd_Paste";
import { cmd_Fill } from "./commands/selection/cmd_Fill";
import { cmd_Rotate } from "./commands/selection/cmd_Rotate";
import { BrushUI } from "./worldFab/BrushUI";
import { SelectionUI } from "./worldFab/SelectionUI";
import { notify } from "./other/helpers";

export let BRUSH;
export const SELECTION = new Selection();
export function setBRUSH(value) {BRUSH = value};
export function getBRUSH() {return typeof BRUSH !== "undefined" ? BRUSH : (() => { throw new Error("No existing brush") })()};

world.afterEvents.worldInitialize.subscribe(_ => {
    initSave();
    addCommands([
        cmd_AutoKillItems,
        cmd_AllowBuildClipping,
        
        cmd_Brush,
        cmd_Shape,
        cmd_Size,
        cmd_Block,
        cmd_Mask,
        cmd_BlockState,
        
        cmd_GenShapeAt,
        cmd_GenShape,

        cmd_Pos,
        cmd_Delete,
        cmd_Copy,
        cmd_Paste,
        cmd_Fill,
        cmd_Rotate,

        cmd_TestArray
    ]);
});

world.afterEvents.entitySpawn.subscribe((eventData) => {
    if (eventData.entity.typeId === "minecraft:item"
    && world.getDynamicProperty("_autoKillItems")) {
        eventData.entity.kill();
    }
});

function initSave() {
    world.getDynamicProperty("_autoKillItems") ?? world.setDynamicProperty("_autoKillItems", true);
    world.getDynamicProperty("_allowBuildClipping") ?? world.setDynamicProperty("_allowBuildClipping", false);
}

world.beforeEvents.itemUse.subscribe((eventData) => {
    const itemID = eventData.itemStack.typeId

    switch (itemID) {
        case "gg:build_brush":
            if (BRUSH)
                BRUSH.attemptBuild();
            else
                notify("No Brush Config Exists. Try !brush command or the Brush Menu.", "err");
            break;
        case "gg:selectmenu":
            try {
                SELECTION.validatePos();
                SelectionUI.showMainMenu(eventData.source, SELECTION);
            }
            catch (err) {
                notify(err.message, "err")
            }
            break;
        case "gg:brushmenu":
            if (BRUSH == undefined) {
                const tempBrush = new Brush(eventData.source, "ball", 5, null, null, null, true);
                setBRUSH(tempBrush);
                BrushUI.showMainMenu(eventData.source, tempBrush);
            }
            else
                BrushUI.showMainMenu(eventData.source, BRUSH);

            break;
    }
});

world.afterEvents.playerInteractWithBlock.subscribe((eventData) => {
    if (!eventData.itemStack)
        return;

    const itemID = eventData.itemStack.typeId;

    switch (itemID) {
        case "gg:select":
            SELECTION.setPos(eventData.player, 2, eventData.block.location);
            break;
    }
});

world.afterEvents.entityHitBlock.subscribe((eventData) => {
    const mainHand = eventData.damagingEntity.getComponent("equippable").getEquipment(EquipmentSlot.Mainhand);
    if (mainHand == undefined)
        return;
    
    const itemID = mainHand.typeId;

    switch (itemID) {
        case "gg:select":
            SELECTION.setPos(eventData.damagingEntity, 1, eventData.hitBlock.location);
            break;
    }
});

system.runInterval(() => {
    if (SELECTION == undefined)
        return

    SELECTION.drawSelection()
}, 20)