import { ActionFormData, ModalFormData } from "@minecraft/server-ui"
import { system, world, EquipmentSlot } from "@minecraft/server";
import { runCommand, notify } from "../other/helpers";



export class BrushUI {
    static showMainMenu(sender, brush) {
        const menu = new ActionFormData()
            .title("Brush Menu")
            .body("Make / Edit your brush")
            .button("New Brush")
            .button("Edit Brush");


        system.run(() => {
            menu.show(sender).then((res) => {
                try {
                    switch (res.selection) {
                        case 0:
                            BrushUI.makeNewBrush(sender, brush);
                            break;
                        case 1:
                            BrushUI.editBrush(sender, brush);
                            break;
                    }
                }
                catch (err) {
                    notify(err.message, "err");
                }
            });
        });
    }

    static async makeNewBrush(sender, brush) {
        const selectedShape = await BrushUI.selectShape(sender);
        if (selectedShape == null)
            return;
        const selectedSize = await BrushUI.selectSize(sender, brush.size);
        if (selectedSize == null)
            return;
        const selectedBlock = await BrushUI.selectBlock(sender);
        if (selectedBlock == null)
            return;

        if (selectedShape != null && selectedSize != null && selectedBlock != null) {
            brush.setShape(selectedShape);
            brush.setSize(selectedSize);
            brush.setBlockPermutation(selectedBlock);
        }
    }

    static async editBrush(sender, brush) {
        const submenu = new ActionFormData()
            .title("Brush Menu")
            .body("Select what to Change")
            .button("Shape")
            .button("Size")
            .button("Block Type")
            .button("Set Mask")
            .button("Remove Mask");

        const res = await submenu.show(sender);

        if (res.canceled)
            return;

        switch (res.selection) {
            case 0:
                const selectedShape = await BrushUI.selectShape(sender);
                if (selectedShape != null)
                    brush.setShape(selectedShape);
                break;
            case 1:
                const selectedSize = await BrushUI.selectSize(sender, brush.size);
                if (selectedSize != null)
                    brush.setSize(selectedSize);
                break;
            case 2:
                const selectedBlock = await BrushUI.selectBlock(sender);
                if (selectedBlock != null)
                    brush.setBlockPermutation(selectedBlock);
                break;
            case 3:
                const maskMenu = new ActionFormData()
                    .title("Brush Menu")
                    .body("Select Mask")
                    .button("Replace Air")
                    .button("Replace Block");
        
                const maskRes = await maskMenu.show(sender);
                    if (maskRes.canceled)
                    return;
                switch (maskRes.selection) {
                    case 0:
                        brush.setMask([{block: "air", blockData: "defaultState"}])
                        break;
                    case 1:
                        const selectedMask = await BrushUI.selectBlock(sender, "mask");
                        if (selectedMask != null)
                            brush.setBlockPermToMask(selectedMask);
                        break;
                }

                break;
            case 4:
                brush.setMask(null);
                break;
        }

        BrushUI.editBrush(sender, brush);
    }

    static async selectSize(sender, brushSize = 5) {
        if (brushSize > 20)
            throw new Error("Brush size is out of range (1-20)");

        const submenu = new ModalFormData()
            .title("Brush Menu")
            .slider("Set Brush Size", 1, 20, 1, brushSize);

        const res = await submenu.show(sender);

        if (res.canceled)
            return null;
    
        return res.formValues[0];
    }
    
    static async selectShape(sender) {
        const submenu = new ActionFormData()
            .title("Brush Menu")
            .body("Select the Brush Shape")
            .button("Ball")
            .button("Cylinder")
            .button("Cone")
            .button("Cube")
            .button("Pattern");

        const res = await submenu.show(sender);

        if (res.canceled)
            return null;

        switch (res.selection) {
            case 0:
                return "ball";
            case 1:
                return "cylinder";
            case 2:
                return "cone";
            case 3:
                return "cube";
            case 4:
                return "pattern";
            default:
                return "cube"
        }
    }

    static async selectBlock(sender) {
        runCommand(sender.dimension, `title ${sender.name} actionbar Pick (punch) a block with Eyedropper`);
        runCommand(sender.dimension, `give ${sender.name} gg:eyedrop 1`);

        return new Promise((resolve, reject) => {
            const blockEventHandler = (eventData) => {
                const mainHand = eventData.damagingEntity.getComponent("equippable").getEquipment(EquipmentSlot.Mainhand);
                if (mainHand === undefined) {
                    return;
                }
    
                const itemID = mainHand.typeId;
    
                if (itemID === "gg:eyedrop") {
                    runCommand(eventData.damagingEntity.dimension, `clear ${eventData.damagingEntity.name} gg:eyedrop`);
                    const blockPerm = eventData.hitBlock.permutation;

                    world.afterEvents.entityHitBlock.unsubscribe(blockEventHandler);
                    resolve(blockPerm);
                }
            };
    
            world.afterEvents.entityHitBlock.subscribe(blockEventHandler);
        });
    }
}