import { EquipmentSlot, MolangVariableMap, Vector } from "@minecraft/server";
import { runCommand, notify, fillBlocks, fillBlocksMasked, vectorToString } from "../other/helpers.js"
import { Stack } from "../other/Stack.js"

const clipboardSize = 16;
let clipboard = undefined;
let currentClipboard = 0;

export class Selection {
    constructor() {
        this.pos1;
        this.pos2;

        this._operatingDimension;

        this.createClipboard();
    }

    validatePos() {
        if (this.pos1 == undefined)
            throw new Error("Position 1 not specified. Try '!pos 1' or use Select Tool.");
        if (this.pos2 == undefined)
            throw new Error("Position 2 not specified. Try '!pos 2' or use Select Tool.");
    }

    deleteSelection(sender) {
        this.validatePos();

        const startX = Math.min(this.pos2.x, this.pos1.x);
        const endX = Math.max(this.pos2.x, this.pos1.x);

        const startZ = Math.min(this.pos2.z, this.pos1.z);
        const endZ = Math.max(this.pos2.z, this.pos1.z);
        const [y1, y2] = [this.pos1.y, this.pos2.y];
        const cleared = (endX - startX + 1) * (endZ - startZ + 1) * (Math.abs(y1 - y2) + 1);

        if (cleared < 32000)
            fillBlocks(sender.dimension, this.pos1, this.pos2, "air");
        else {
            notify(`Clearing large area, please wait...`, "warn");


            for (let x = startX; x <= endX; x += 2) {
                for (let z = startZ; z <= endZ; z += 2) {
                    fillBlocks(sender.dimension, new Vector(x - 1, y1, z - 1), new Vector(x, y2, z), "air");
                    if (x + 1 == endX)
                        fillBlocks(sender.dimension, new Vector(x, y1, z), new Vector(x + 1, y2, z), "air");
                    if (z + 1 == endZ)
                        fillBlocks(sender.dimension, new Vector(x, y1, z), new Vector(x, y2, z + 1), "air");
                }
            }

        }

        notify(`Cleared area of ${cleared} blocks, between §9[${this.pos1.x}, ${this.pos1.y}, ${this.pos1.z}]§r and §2[${this.pos1.x}, ${this.pos1.y}, ${this.pos1.z}]§r`);

    }


    fillSelection(sender, block, mask = null) {
        this.validatePos();

        const startX = Math.min(this.pos2.x, this.pos1.x);
        const endX = Math.max(this.pos2.x, this.pos1.x);

        const startZ = Math.min(this.pos2.z, this.pos1.z);
        const endZ = Math.max(this.pos2.z, this.pos1.z);
        const [y1, y2] = [this.pos1.y, this.pos2.y];
        const cleared = (endX - startX + 1) * (endZ - startZ + 1) * (Math.abs(y1 - y2) + 1);

        try {
            if (cleared < 32000) {
                if (mask == null) {
                    fillBlocks(sender.dimension,
                                this.pos1,
                                this.pos2,
                                block);
                }
                else {
                    fillBlocksMasked(sender.dimension,
                                    this.pos1,
                                    this.pos2,
                                    block,
                                    mask);
                }
            }
            else {
                notify(`Filling large area, please wait...`, "warn");


                for (let x = startX; x <= endX; x += 2) {
                    for (let z = startZ; z <= endZ; z += 2) {
                        if (mask == null) {
                            fillBlocks(sender.dimension, new Vector(x - 1, y1, z - 1), new Vector(x, y2, z), block);
                            if (x + 1 == endX)
                                fillBlocks(sender.dimension, new Vector(x, y1, z), new Vector(x + 1, y2, z), block);
                            if (z + 1 == endZ)
                                fillBlocks(sender.dimension, new Vector(x, y1, z), new Vector(x, y2, z + 1), block);
                        }
                        else {
                            fillBlocksMasked(sender.dimension, new Vector(x - 1, y1, z - 1), new Vector(x, y2, z), block, mask);
                            if (x + 1 == endX)
                            fillBlocksMasked(sender.dimension, new Vector(x, y1, z), new Vector(x + 1, y2, z), block, mask);
                            if (z + 1 == endZ)
                            fillBlocksMasked(sender.dimension, new Vector(x, y1, z), new Vector(x, y2, z + 1), block, mask);
                        }
                    }
                }

            }
        }
        catch (err) {
            notify("Selection out of range / block type does not exist", "err");
        }
    }

    copySelection(sender) {
        this.validatePos();
        try {
            currentClipboard++;
            let name = `item_${currentClipboard}`;
            clipboard.add(name);
            runCommand(sender.dimension, `structure save ${name} ${vectorToString(this.pos1)} ${vectorToString(this.pos2)} false memory`);
        }
        catch {
            notify("Selection out of range or positions not specified", "error");
        }
    }

    pasteSelection(sender, index = 0) {
        this.validatePos();
        if (clipboard.isEmpty())
            return;

        let name = `item_${currentClipboard - index}`;
        runCommand(sender.dimension, `execute at @a[name=${sender.name}] run structure load ${name} ~ ~ ~`);
    }

    rotateSelection(sender, rotation) {
        this.validatePos();
        if (rotation == "90" || rotation == "180" || rotation == "270") {
            // TODO rotate around center
            // Sorta works
            // let tempStructure = "temp"
            // runCommand(sender.dimension, `structure save ${tempStructure} ${vectorToString(this.pos1)} ${vectorToString(this.pos2)} false memory`)
            // fillBlocks(sender.dimension, this.pos1, this.pos2, "air")
            
            // const longestAxis = Math.abs(Math.abs(this.pos1.x) - Math.abs(this.pos2.x)) > Math.abs(Math.abs(this.pos1.z) - Math.abs(this.pos2.z)) ? "x" : "z"
            // const loadPos = this.pos1.y < this.pos2.y ? this.pos1 : this.pos2

            // const centerX = Math.round((this.pos1.x + this.pos2.x) / 2);
            // const centerZ = Math.round((this.pos1.z + this.pos2.z) / 2);


            // if (rotation == "180") {
            //     const offsetZ = longestAxis == "z" ? Math.abs(Math.abs(this.pos1.z) - Math.abs(this.pos2.z)) / 2 : 0
            //     const offsetX = longestAxis == "x" ? Math.abs(Math.abs(this.pos1.x) - Math.abs(this.pos2.x)) / 2 : 0
            //     runCommand(sender.dimension, `structure load ${tempStructure} ${centerX - offsetX} ${loadPos.y} ${centerZ - offsetZ} ${rotation}_degrees`)
            // }
            // else {
            //     const offsetX = longestAxis == "z" ? Math.abs(Math.abs(this.pos1.z) - Math.abs(this.pos2.z)) / 2 : 0
            //     const offsetZ = longestAxis == "x" ? Math.abs(Math.abs(this.pos1.x) - Math.abs(this.pos2.x)) / 2 : 0
            //     runCommand(sender.dimension, `structure load ${tempStructure} ${centerX - offsetX} ${loadPos.y} ${centerZ - offsetZ} ${rotation}_degrees`)
            // }

            let tempStructure = "temp";
            const lowestY = Math.min(this.pos1.y, this.pos2.y);
            runCommand(sender.dimension, `structure save ${tempStructure} ${vectorToString(this.pos1)} ${vectorToString(this.pos2)} false memory`);
            fillBlocks(sender.dimension, this.pos1, this.pos2, "air");
            runCommand(sender.dimension, `structure load ${tempStructure} ${this.pos1.x} ${lowestY} ${ this.pos1.z} ${rotation}_degrees`);
        }
        else {
            notify(`${rotation} is not a valid rotation. Do !help or !h for help`, "error");
        }
    }

    mirrorSelection() {
        // TODO
    }

    setPos(sender, type, coordinate) {
        if (sender.dimension != this._operatingDimension) {
            this._operatingDimension = sender.dimension;
        }

        switch (parseInt(type)) {
            case 1:
                this.pos1 = coordinate;
                break;
            case 2:
                this.pos2 = coordinate;
                break;
            default:
                notify("Type not recognised", "err");
                break;
        }

        notify(`Position ${type} set to §2[${coordinate.x}, ${coordinate.y}, ${coordinate.z}]§r`);
    }


    createClipboard() {
        clipboard = new Stack(clipboardSize);
    }

    shouldDraw() {
        if (this._operatingDimension == undefined)
            return false;

        const allPlayers = this._operatingDimension.getPlayers();

        for (const p of allPlayers) {
            const mainHand = p.getComponent("equippable").getEquipment(EquipmentSlot.Mainhand);
            if (mainHand == undefined)
                return;
            
            const itemID = mainHand.typeId;
            if (itemID == "gg:select" || itemID == "gg:selectmenu")
                return true;
        }

        return false;
    }

    drawSelection() {
        if (this._operatingDimension == undefined || !this.shouldDraw())
            return;
            

        try {
            this.validatePos();
            // TODO BRUH, MY GUY, WHY DOES THE VARIABLE NOT CHANGE
            // try {
            //     const mvm = new MolangVariableMap();
            //     mvm.setFloat("variable.markerType", 1.0);
            //     this._operatingDimension.spawnParticle("gg:marker", this.pos1, mvm);
            // }
            // catch (err) {console.log(err)}
            // try {
            //     this._operatingDimension.spawnParticle("gg:marker", this.pos2, new MolangVariableMap().setFloat("variable.markerType", 2.0));
            // }
            // catch {}

            const startX = Math.min(this.pos1.x, this.pos2.x);
            const endX = Math.max(this.pos1.x, this.pos2.x);
            const startY = Math.min(this.pos1.y, this.pos2.y);
            const endY = Math.max(this.pos1.y, this.pos2.y);
            const startZ = Math.min(this.pos1.z, this.pos2.z);
            const endZ = Math.max(this.pos1.z, this.pos2.z);

            // Spawn particles on the outline of the cube
            for (let x = startX; x <= endX; x += 2) {
                try {
                    this._operatingDimension.spawnParticle("gg:marker", new Vector(x, startY, startZ));
                    this._operatingDimension.spawnParticle("gg:marker", new Vector(x, startY, endZ));
                    this._operatingDimension.spawnParticle("gg:marker", new Vector(x, endY, startZ));
                    this._operatingDimension.spawnParticle("gg:marker", new Vector(x, endY, endZ));
                } 
                catch {}
            }
        
            for (let y = startY + 1; y < endY; y += 2) {
                try {
                    this._operatingDimension.spawnParticle("gg:marker", new Vector(startX, y, startZ));
                    this._operatingDimension.spawnParticle("gg:marker", new Vector(startX, y, endZ));
                    this._operatingDimension.spawnParticle("gg:marker", new Vector(endX, y, startZ));
                    this._operatingDimension.spawnParticle("gg:marker", new Vector(endX, y, endZ));
                } 
                catch {}
            }
        
            for (let z = startZ + 1; z < endZ; z += 2) {
                try {
                    this._operatingDimension.spawnParticle("gg:marker", new Vector(startX, startY, z));
                    this._operatingDimension.spawnParticle("gg:marker", new Vector(startX, endY, z));
                    this._operatingDimension.spawnParticle("gg:marker", new Vector(endX, startY, z));
                    this._operatingDimension.spawnParticle("gg:marker", new Vector(endX, endY, z));
                } 
                
                catch {}
            }
        }
        catch {}
    }
}