import { ActionFormData } from "@minecraft/server-ui"
import { system, world, EquipmentSlot } from "@minecraft/server";

import { runCommand, notify, formatObject } from "../other/helpers.js"

export class SelectionUI {
    static showMainMenu(sender, selection) {
        const menu = new ActionFormData()
            .title("Selection Menu")
            .body("Pick what to do with your selection")
            .button("Delete")
            .button("Fill with:")
            .button("Copy")
            .button("Paste")
            .button("Rotate");


        system.run(() => {
            menu.show(sender).then((res) => {
                try {
                    switch (res.selection) {
                        case 0:
                            selection.deleteSelection(sender);
                            break;
                        case 1:
                            SelectionUI.chooseFill(sender, selection);
                            break;
                        case 2:
                            selection.copySelection(sender);
                            break;
                        case 3:
                            selection.pasteSelection(sender);
                            break;
                        case 4:
                            SelectionUI.rotateMenu(sender, selection);
                            break;
                    }
                }
                catch (err) {
                    notify(err.message, "err");
                }
            });
        });
    }

    static async chooseFill(sender, selection) {
        const selectedBlock = await SelectionUI.selectBlock(sender);
        if (selectedBlock != null) {
            SelectionUI.confirmFill(sender, selection, selectedBlock);
        }
    }

    static async chooseMask(sender, selection, fillBlock) {
        const selectedMask = await SelectionUI.selectBlock(sender);

        if (selectedMask != null) {
            SelectionUI.confirmMask(sender, selection, fillBlock, selectedMask);
        }
    }

    static async selectBlock(sender) {
        runCommand(sender.dimension, `title ${sender.name} actionbar Pick (punch) a block with Eyedropper`);
        runCommand(sender.dimension, `give ${sender.name} gg:eyedrop 1`);

        return new Promise((resolve, reject) => {
            const blockEventHandler = (eventData) => {
                const mainHand = eventData.damagingEntity.getComponent("equippable").getEquipment(EquipmentSlot.Mainhand);
                if (mainHand === undefined) {
                    return;
                }
    
                const itemID = mainHand.typeId;
    
                if (itemID === "gg:eyedrop") {
                    runCommand(eventData.damagingEntity.dimension, `clear ${eventData.damagingEntity.name} gg:eyedrop`);
                    const blockPerm = eventData.hitBlock.permutation;

                    world.afterEvents.entityHitBlock.unsubscribe(blockEventHandler);
                    resolve(blockPerm);
                }
            };
    
            world.afterEvents.entityHitBlock.subscribe(blockEventHandler);
        });
    }

    static confirmFill(sender, selection, eyedropBlock) {
        const fillBlock = eyedropBlock
        const submenu = new ActionFormData()
            .title("Selection Menu")
            .body(`Fill your selection with ${fillBlock.type.id} ${formatObject(fillBlock.getAllStates())}`)
            .button("Yes")
            .button("No, apply a mask first")
            .button("No, pick a different block");

        system.run(() => {
            submenu.show(sender).then((res) => {
                switch (res.selection) {
                    case 0:
                        selection.fillSelection(sender, fillBlock);
                        break
                    case 1:
                        SelectionUI.chooseMask(sender, selection, fillBlock);
                        break
                    case 2:
                        SelectionUI.chooseFill(sender, selection);
                        break
                    default:
                        break
                }
            });
        });
        
    }

    static confirmMask(sender, selection, fillBlock, eyedropBlock) {
        const maskBlock = eyedropBlock;
        // masking excludes block types as fillSelection can't process block states yet
        const submenu = new ActionFormData()
            .title("Selection Menu")
            .body(`Fill all ${maskBlock.type.id} with ${fillBlock.type.id} ${formatObject(fillBlock.getAllStates())}`)
            .button("Yes")
            .button("No");

        system.run(() => {
            submenu.show(sender).then((res) => {
                switch (res.selection) {
                    case 0:
                        let mask = [{
                            block: maskBlock.type.id,
                            blockData: maskBlock.getAllStates()
                        }];
                        selection.fillSelection(sender, fillBlock, mask);
                        break;
                }
            });
        });
    }

    static rotateMenu(sender, selection) {
        const submenu = new ActionFormData()
            .title("Selection Menu")
            .body("Rotate your selection")
            .button("Rotate 90°")
            .button("Rotate 180°")
            .button("Rotate 270°");


        system.run(() => {
            submenu.show(sender).then((res) => {
                switch (res.selection) {
                    case 0:
                        selection.rotateSelection(sender, "90");
                        break;
                    case 1:
                        selection.rotateSelection(sender, "180");
                        break;
                    case 2:
                        selection.rotateSelection(sender, "270");
                        break;
                }
            });
        });
    }
}